@@ -22,7 +22,6 @@
 #define GLOBAL_CFG		VZ_DIR "vz.conf"
 #define OSRELEASE_CFG		VZ_DIR "osrelease.conf"
 #define VPS_CONF_DIR		VZ_DIR "conf/"
-#define LIB_SCRIPTS_DIR		PKGLIBDIR "/scripts/"
 #define DIST_DIR		VZ_DIR "dists"
 #define VENAME_DIR		VZ_DIR "names"
 
@@ -30,9 +29,11 @@
 
 #define VZFIFO_FILE		"/.vzfifo"
 
-#define VPS_NET_ADD		LIB_SCRIPTS_DIR "vps-net_add"
-#define VPS_NET_DEL		LIB_SCRIPTS_DIR "vps-net_del"
-#define VPS_PCI			LIB_SCRIPTS_DIR "vps-pci"
+#define VPS_NET_ADD		SCRIPTDIR "/vps-net_add"
+#define VPS_NET_DEL		SCRIPTDIR "/vps-net_del"
+#define VPS_NETNS_DEV_ADD	SCRIPTDIR "/vps-netns_dev_add"
+#define VPS_NETNS_DEV_DEL	SCRIPTDIR "/vps-netns_dev_del"
+#define VPS_PCI			SCRIPTDIR "/vps-pci"
 
 /* User-defined scripts are in VPS_CONF_DIR */
 #define USER_CREATE_SCRIPT	VPS_CONF_DIR "vps.create"
@@ -80,13 +81,39 @@ enum {
 #define ERR_INVAL_SKIP	-6
 #define ERR_LONG_TRUNC  -7
 
+struct arg_start;
+struct fs_param;
+struct arg_start;
+struct ub_struct;
+struct dev_res;
+struct cpu_param;
+struct veth_dev;
+struct meminfo_param;
+
 /** CT handler.
  */
-typedef struct {
+typedef struct vps_handler {
 	int vzfd;	/**< /dev/vzctl file descriptor. */
 	int stdfd;
+	int can_join_pidns; /* can't enter otherwise */
+	int (*is_run)(struct vps_handler *h, envid_t veid);
+	int (*enter)(struct vps_handler *h, envid_t veid, const char *root, int flags);
+	int (*destroy)(struct vps_handler *h, envid_t veid);
+	int (*env_create)(struct arg_start *arg);
+	int (*setlimits)(struct vps_handler *h, envid_t, struct ub_struct *ub);
+	int (*setcpus)(struct vps_handler *h, envid_t, struct cpu_param *cpu);
+	int (*setcontext)(envid_t veid);
+	int (*setdevperm)(struct vps_handler *h, envid_t veid, struct dev_res *res);
+	int (*netdev_ctl)(struct vps_handler *h, envid_t veid, int op, char *name);
+	int (*ip_ctl)(struct vps_handler *h, envid_t veid, int op, const char *ipstr);
+	int (*veth_ctl)(struct vps_handler *h, envid_t veid, int op, struct veth_dev *veth);
 } vps_handler;
 
+static inline int is_vz_kernel(vps_handler *h)
+{
+	return h->vzfd != -1;
+}
+
 typedef enum {
 	SKIP_NONE =		0,
 	SKIP_SETUP =		(1<<0),
