@@ -40,11 +40,14 @@ typedef enum {
 	ACTION_QUOTAOFF,
 	ACTION_QUOTAINIT,
 	ACTION_CONSOLE,
+#ifdef HAVE_PLOOP
 	ACTION_CONVERT,
 	ACTION_SNAPSHOT_CREATE,
 	ACTION_SNAPSHOT_SWITCH,
 	ACTION_SNAPSHOT_DELETE,
 	ACTION_SNAPSHOT_LIST,
+	ACTION_COMPACT,
+#endif
 } act_t;
 
 /* default cpu units values */
