@@ -70,7 +70,6 @@
 #define PARAM_QUOTAUGIDLIMIT	39
 #define PARAM_DISK_QUOTA	40
 
-#define PARAM_NOATIME		43
 #define PARAM_SAFE		44
 #define PARAM_CAP		45
 #define PARAM_CONFIG		47
@@ -146,6 +145,7 @@
 #define PARAM_SNAPSHOT_SKIP_SUSPEND	373
 
 #define PARAM_SKIP_UMOUNT	374
+#define PARAM_MOUNT_OPTS	396
 
 #define PARAM_LINE		"e:p:f:t:i:l:k:a:b:n:x:h"
 #endif
