@@ -37,7 +37,7 @@
 #include "logger.h"
 #include "list.h"
 #include "bitmap.h"
-#include "config.h"
+#include "vzconfig.h"
 #include "vzctl_param.h"
 #include "vzerror.h"
 #include "util.h"
@@ -112,7 +112,7 @@ static vps_config config[] = {
 {"VE_ROOT",	NULL, PARAM_ROOT},
 {"VE_PRIVATE",	NULL, PARAM_PRIVATE},
 {"TEMPLATE",	NULL, PARAM_TEMPLATE},
-{"NOATIME",	NULL, PARAM_NOATIME},
+{"MOUNT_OPTS",	NULL, PARAM_MOUNT_OPTS},
 {"VE_LAYOUT",	NULL, PARAM_VE_LAYOUT},
 /*	template     */
 {"OSTEMPLATE",	NULL, PARAM_OSTEMPLATE},
@@ -158,6 +158,7 @@ static vps_config config[] = {
 {"IPTABLES_MODULES",	NULL, PARAM_IGNORED},
 {"IP6TABLES",		NULL, PARAM_IGNORED},
 {"CONFIG_CUSTOMIZED",	NULL, PARAM_IGNORED},
+{"NOATIME",		NULL, PARAM_IGNORED},
 
 {NULL,		NULL, -1}
 };
@@ -928,8 +929,8 @@ static int store_fs(vps_param *old_p, vps_param *vps_p, vps_config *conf,
 	case PARAM_PRIVATE:
 		ret = conf_store_str(conf_h, conf->name, fs->private_orig);
 		break;
-	case PARAM_NOATIME:
-		ret = conf_store_yesno(conf_h, conf->name, fs->noatime);
+	case PARAM_MOUNT_OPTS:
+		ret = conf_store_str(conf_h, conf->name, fs->mount_opts);
 		break;
 	}
 	return ret;
@@ -2048,9 +2049,6 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	case PARAM_TEMPLATE:
 		ret = conf_parse_str(&vps_p->res.fs.tmpl, val);
 		break;
-	case PARAM_NOATIME:
-		ret = conf_parse_yesno(&vps_p->res.fs.noatime, val);
-		break;
 	case PARAM_VE_LAYOUT:
 		ret = parse_ve_layout(&vps_p->opt.layout,
 				&vps_p->opt.mode, val);
@@ -2195,6 +2193,9 @@ static int parse(envid_t veid, vps_param *vps_p, char *val, int id)
 	case PARAM_SNAPSHOT_SKIP_SUSPEND:
 		vps_p->snap.flags |= SNAPSHOT_SKIP_SUSPEND;
 		break;
+	case PARAM_MOUNT_OPTS:
+		ret = conf_parse_str(&vps_p->res.fs.mount_opts, val);
+		break;
 	case PARAM_IGNORED:
 		/* Well known but ignored parameter */
 		break;
@@ -2329,6 +2330,36 @@ int vps_parse_config(envid_t veid, char *path, vps_param *vps_p,
 	return err;
 }
 
+vps_param *reread_vps_config(envid_t veid)
+{
+	vps_param *gparam = NULL;
+	vps_param *vps_p = NULL;
+	char fname[PATH_MAX];
+
+	get_vps_conf_path(veid, fname, sizeof(fname));
+	if (stat_file(fname))
+		goto err;
+
+	gparam = init_vps_param();
+	if (vps_parse_config(veid, GLOBAL_CFG, gparam, NULL))
+		goto err;
+
+	vps_p = init_vps_param();
+	if (vps_parse_config(veid, fname, vps_p, NULL))
+		goto err;
+
+	merge_vps_param(gparam, vps_p);
+	free_vps_param(vps_p);
+
+	return gparam;
+
+err:
+	free_vps_param(gparam);
+	free_vps_param(vps_p);
+
+	return NULL;
+}
+
 /********************************************************/
 /*	CT save config stuff				*/
 /********************************************************/
@@ -2629,6 +2660,7 @@ static void free_fs(fs_param *fs)
 	FREE_P(fs->private)
 	FREE_P(fs->private_orig)
 	FREE_P(fs->tmpl)
+	FREE_P(fs->mount_opts)
 }
 
 static void free_tmpl(tmpl_param *tmpl)
@@ -2777,7 +2809,8 @@ static void merge_fs(fs_param *dst, fs_param *src)
 	MERGE_STR(private)
 	MERGE_STR(private_orig)
 	MERGE_STR(tmpl)
-	MERGE_INT(noatime)
+	MERGE_STR(mount_opts)
+	MERGE_INT(flags)
 }
 
 static void merge_tmpl(tmpl_param *dst, tmpl_param *src)
