@@ -31,7 +31,6 @@
 #include <linux/vzcalluser.h>
 #include <string.h>
 #include <sys/types.h>
-#include <sys/param.h>
 #include <unistd.h>
 #include <dirent.h>
 
@@ -40,7 +39,7 @@
 #include "env.h"
 #include "exec.h"
 #include "script.h"
-#include "config.h"
+#include "vzconfig.h"
 #include "vzerror.h"
 #include "logger.h"
 #include "util.h"
@@ -266,7 +265,7 @@ int vps_chkpnt(vps_handler *h, envid_t veid, const fs_param *fs,
 		ret = VZ_RESOURCE_ERROR;
 		goto err;
 	} else if (pid == 0) {
-		if ((ret = vz_setluid(veid)))
+		if ((ret = h->setcontext(veid)))
 			exit(ret);
 		if ((pid = fork()) < 0) {
 			logger(-1, errno, "Can't fork");
@@ -293,8 +292,11 @@ err:
 			if (dumpfile)
 				unlink(dumpfile);
 	}
-	if (dump_fd != -1)
+	if (dump_fd != -1) {
+		if (ret == 0)
+			fsync(dump_fd);
 		close(dump_fd);
+	}
 	if (cpt_fd != -1)
 		close(cpt_fd);
 
@@ -313,7 +315,7 @@ static int restore_fn(vps_handler *h, envid_t veid, int wait_p,
 	if (param == NULL)
 		goto err;
 	/* Close all fds */
-	close_fds(0, wait_p, old_wait_p, err_p, h->vzfd, param->rst_fd, -1);
+	close_fds(0, wait_p, old_wait_p, err_p, param->rst_fd, h->vzfd, -1);
 	if (ioctl(param->rst_fd, CPT_SET_VEID, veid) < 0) {
 		logger(-1, errno, "Can't set CT ID %d", param->rst_fd);
 		goto err;
