@@ -16,17 +16,15 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include <grp.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
 #include <stdio.h>
 #include <signal.h>
 #include <fcntl.h>
-#include <sys/types.h>
 #include <sys/wait.h>
 #include <string.h>
-#include <grp.h>
-#include <sys/ioctl.h>
 #include <linux/vzcalluser.h>
 #include <sys/personality.h>
 #include <linux/reboot.h>
@@ -42,91 +40,14 @@
 #include "util.h"
 #include "script.h"
 #include "iptables.h"
-#include "readelf.h"
 #include "vzsyscalls.h"
 #include "cpt.h"
 #include "image.h"
-
-#define ENVRETRY	3
+#include "readelf.h"
 
 static int env_stop(vps_handler *h, envid_t veid, const char *root,
 		int stop_mode);
 
-static inline int setluid(uid_t uid)
-{
-	return syscall(__NR_setluid, uid);
-}
-
-#ifdef  __x86_64__
-static int set_personality(unsigned long mask)
-{
-	unsigned long per;
-
-	per = personality(0xffffffff) | mask;
-	logger(3, 0, "Set personality %#10.8lx", per);
-	if (personality(per) == -1) {
-		logger(-1, errno, "Unable to set personality PER_LINUX32");
-		return  -1;
-	}
-	return 0;
-}
-
-static int set_personality32()
-{
-	if (get_arch_from_elf("/sbin/init") != elf_32)
-		return 0;
-	return set_personality(PER_LINUX32);
-}
-#endif
-
-static int vz_env_create_data_ioctl(vps_handler *h,
-	struct vzctl_env_create_data *data)
-{
-	int errcode;
-	int retry = 0;
-
-	do {
-		if (retry)
-			sleep(1);
-		errcode = ioctl(h->vzfd, VZCTL_ENV_CREATE_DATA, data);
-	} while (errcode < 0 && errno == EBUSY && retry++ < ENVRETRY);
-
-	if (errcode >= 0) {
-		/* Clear supplementary group IDs */
-		setgroups(0, NULL);
-#ifdef  __x86_64__
-		/* Set personality PER_LINUX32 for i386 based CTs */
-		set_personality32();
-#endif
-	}
-	return errcode;
-}
-
-int vz_env_create_ioctl(vps_handler *h, envid_t veid, int flags)
-{
-	struct vzctl_env_create env_create;
-	int errcode;
-	int retry = 0;
-
-	memset(&env_create, 0, sizeof(env_create));
-	env_create.veid = veid;
-	env_create.flags = flags;
-	do {
-		if (retry)
-			sleep(1);
-		errcode = ioctl(h->vzfd, VZCTL_ENV_CREATE, &env_create);
-	} while (errcode < 0 && errno == EBUSY && retry++ < ENVRETRY);
-	if (errcode >= 0 && (flags & VE_ENTER)) {
-		/* Clear supplementary group IDs */
-		setgroups(0, NULL);
-#ifdef  __x86_64__
-		/* Set personality PER_LINUX32 for i386 based CTs */
-		set_personality32();
-#endif
-	}
-	return errcode;
-}
-
 /*
  * Reset standard file descriptors to /dev/null in case they are closed.
  */
@@ -155,37 +76,35 @@ static int reset_std()
  */
 vps_handler *vz_open(envid_t veid)
 {
-	int vzfd, stdfd;
 	vps_handler *h = NULL;
+	int ret = -1;
 
-	stdfd = reset_std();
-	if ((vzfd = open(VZCTLDEV, O_RDWR)) < 0) {
-		logger(-1, errno, "Unable to open %s", VZCTLDEV);
-		logger(-1, 0, "Please check that vzdev kernel module is loaded"
-			" and you have sufficient permissions"
-			" to access the file.");
-		goto err;
-	}
 	h = calloc(1, sizeof(*h));
 	if (h == NULL)
-		goto err;
-	h->vzfd = vzfd;
-	h->stdfd = stdfd;
-	if (vz_env_create_ioctl(h, 0, 0) < 0 &&
-		(errno == ENOSYS || errno == EPERM))
-	{
-		logger(-1, 0, "Your kernel lacks support for virtual"
-			" environments or modules not loaded");
-		goto err;
-	}
-	return h;
+		return NULL;
 
-err:
+	h->stdfd = reset_std();
+
+	/* Find out if we are under OpenVZ or upstream kernel */
+	if (stat_file("/proc/vz"))
+		ret = vz_do_open(h);
+	else {
+		logger(0, 0, "Directory /proc/vz not found, assuming "
+				"non-OpenVZ kernel");
+		h->vzfd = -1;
+#ifdef HAVE_UPSTREAM
+		ret = ct_do_open(h);
+#else
+		logger(-1, 0, "Support for non-OpenVZ kernel not compiled in");
+#endif
+	}
+
+	if (!ret)
+		return h;
+
+	if (h->stdfd != -1)
+		close(h->stdfd);
 	free(h);
-	if (vzfd != -1)
-		close(vzfd);
-	if (stdfd != -1)
-		close(stdfd);
 	return NULL;
 }
 
@@ -212,17 +131,9 @@ void vz_close(vps_handler *h)
  */
 int vps_is_run(vps_handler *h, envid_t veid)
 {
-	int ret;
-
 	if (veid == 0)
 		return 1;
-	ret = vz_env_create_ioctl(h, veid, VE_TEST);
-
-	if (ret < 0 && (errno == ESRCH || errno == ENOTTY))
-		return 0;
-	else if (ret < 0)
-		logger(-1, errno, "error on vz_env_create_ioctl(VE_TEST)");
-	return 1;
+	return h->is_run(h, veid);
 }
 
 /** Change root to specified directory
@@ -261,43 +172,6 @@ int vz_chroot(const char *root)
 	return 0;
 }
 
-int vz_setluid(envid_t veid)
-{
-	if (setluid(veid) == -1) {
-		if (errno == ENOSYS)
-			logger(-1, 0, "Error: kernel does not support"
-				" user resources. Please, rebuild with"
-				" CONFIG_USER_RESOURCE=y");
-		return VZ_SETLUID_ERROR;
-	}
-	return 0;
-}
-
-static int vz_env_configure(int fd, envid_t veid, const char *osrelease)
-{
-	int ret = 0;
-	struct vzctl_ve_configure *cparam;
-	int len;
-
-	len = strlen(osrelease) + 1;
-	cparam = calloc(1, sizeof(struct vzctl_ve_configure) + len);
-	if (cparam == NULL)
-		return VZ_RESOURCE_ERROR;
-
-	cparam->veid = veid;
-	cparam->key = VE_CONFIGURE_OS_RELEASE;
-	cparam->size = len;
-	strcpy(cparam->data, osrelease);
-
-	if (ioctl(fd, VZCTL_VE_CONFIGURE, cparam) != 0)
-		if (errno != ENOTTY)
-			ret = VZ_SET_OSRELEASE;
-
-	free(cparam);
-
-	return ret;
-}
-
 static int configure_sysctl()
 {
 	int fd;
@@ -310,96 +184,70 @@ static int configure_sysctl()
 	return 0;
 }
 
-static int _env_create(vps_handler *h, envid_t veid, int wait_p, int err_p,
-	void *data)
+static int set_personality(unsigned long mask)
+{
+	unsigned long per;
+
+	per = personality(0xffffffff) | mask;
+	logger(3, 0, "Set personality %#10.8lx", per);
+	if (personality(per) == -1) {
+		logger(-1, errno, "Unable to set personality PER_LINUX32");
+		return  -1;
+	}
+	return 0;
+}
+
+int set_personality32()
+{
+	if (get_arch_from_elf("/sbin/init") != elf_32)
+		return 0;
+	return set_personality(PER_LINUX32);
+}
+
+void fill_container_param(struct arg_start *arg,
+			 struct env_create_param3 *create_param)
+{
+	memset(create_param, 0, sizeof(*create_param));
+	create_param->iptables_mask = get_ipt_mask(arg->res->env.ipt_mask);
+	logger(3, 0, "Setting iptables mask %#10.8llx",
+			(unsigned long long) create_param->iptables_mask);
+	clean_hardlink_dir("/");
+	if (arg->res->cpu.vcpus != NULL)
+		create_param->total_vcpus = *arg->res->cpu.vcpus;
+
+	create_param->feature_mask = arg->res->env.features_mask;
+	create_param->known_features = arg->res->env.features_known;
+
+	/* sysfs enabled by default, unless explicitly disabled */
+	if (! (arg->res->env.features_known & VE_FEATURE_SYSFS)) {
+		create_param->feature_mask |= VE_FEATURE_SYSFS;
+		create_param->known_features |= VE_FEATURE_SYSFS;
+	}
+	logger(3, 0, "Setting features mask %016llx/%016llx",
+			create_param->feature_mask,
+			create_param->known_features);
+}
+
+int exec_container_init(struct arg_start *arg,
+			struct env_create_param3 *create_param)
 {
-	struct vzctl_env_create_data env_create_data;
-	struct env_create_param3 create_param;
 	int fd, ret;
-	vps_res *res;
 	char *argv[] = {"init", "-z", "      ", NULL};
 	char *envp[] = {"HOME=/", "TERM=linux", NULL};
 
-	res = (vps_res *) data;
-	memset(&create_param, 0, sizeof(create_param));
-	create_param.iptables_mask = get_ipt_mask(res->env.ipt_mask);
-	logger(3, 0, "Set iptables mask %#10.8llx",
-			(unsigned long long) create_param.iptables_mask);
-	clean_hardlink_dir("/");
-	if (res->cpu.vcpus != NULL)
-		create_param.total_vcpus = *res->cpu.vcpus;
-	env_create_data.veid = veid;
-	env_create_data.class_id = 0;
-	env_create_data.flags = VE_CREATE | VE_EXCLUSIVE;
-	env_create_data.data = &create_param;
-	env_create_data.datalen = sizeof(create_param);
+	/* Clear supplementary group IDs */
+	setgroups(0, NULL);
+	/* for 32-bit userspace running over 64-bit kernels */
+	set_personality32();
 
-	create_param.feature_mask = res->env.features_mask;
-	create_param.known_features = res->env.features_known;
-	/* sysfs enabled by default, unless explicitly disabled */
-	if (! (res->env.features_known & VE_FEATURE_SYSFS)) {
-		create_param.feature_mask |= VE_FEATURE_SYSFS;
-		create_param.known_features |= VE_FEATURE_SYSFS;
-	}
-	logger(3, 0, "Set features mask %016llx/%016llx",
-			create_param.feature_mask,
-			create_param.known_features);
-
-	/* Close all fds except stdin. stdin is status pipe */
-	close(STDERR_FILENO); close(STDOUT_FILENO);
-	close_fds(0, wait_p, err_p, h->vzfd, -1);
-try:
-	ret = vz_env_create_data_ioctl(h, &env_create_data);
-	if (ret < 0) {
-		switch(errno) {
-			case EINVAL:
-				ret = VZ_ENVCREATE_ERROR;
-				/* Run-time kernel did not understand the
-				 * latest create_param -- so retry with
-				 * the old env_create_param structs.
-				 */
-				switch (env_create_data.datalen) {
-				case sizeof(struct env_create_param3):
-					env_create_data.datalen =
-						sizeof(struct env_create_param2);
-					goto try;
-				case sizeof(struct env_create_param2):
-					env_create_data.datalen =
-						sizeof(struct env_create_param);
-					goto try;
-				}
-				break;
-			case EACCES:
-			/* License is not loaded */
-				ret = VZ_NO_ACCES;
-				break;
-			case ENOTTY:
-			/* Some vz modules are not present */
-				ret = VZ_BAD_KERNEL;
-				break;
-			default:
-				logger(-1, errno, "env_create error");
-				ret = VZ_ENVCREATE_ERROR;
-				break;
-		}
-		goto env_err;
-	}
-	if (res->env.osrelease != NULL) {
-		ret = vz_env_configure(h->vzfd, veid,
-				res->env.osrelease);
-		if (ret != 0)
-			goto env_err;
-	}
-
-	close(h->vzfd);
 	/* Create /fastboot to skip run fsck */
 	fd = open("/fastboot", O_CREAT | O_RDONLY, 0644);
 	close(fd);
 
-	if (res->misc.wait == YES) {
+	if (arg->res->misc.wait == YES) {
 		if (add_reach_runlevel_mark()) {
 			ret = VZ_WAIT_FAILED;
-			goto env_err;
+			return -1;
 		}
 	}
 
@@ -407,7 +255,7 @@ try:
 	if (stat_file("/sys"))
 		mount("sysfs", "/sys", "sysfs", 0, 0);
 
-	if (create_param.feature_mask & VE_FEATURE_NFSD) {
+	if (create_param->feature_mask & VE_FEATURE_NFSD) {
 		mount("nfsd", "/proc/fs/nfsd", "nfsd", 0, 0);
 		make_dir("/var/lib/nfs/rpc_pipefs", 1);
 		mount("sunrpc", "/var/lib/nfs/rpc_pipefs", "rpc_pipefs", 0, 0);
@@ -421,53 +269,41 @@ try:
 	/* Now we wait until CT setup will be done
 	   If no error, then start init, otherwise exit.
 	*/
-	if (read(wait_p, &ret, sizeof(ret)) == 0)
-		return 0;
+
+	if (read(arg->wait_p, &ret, sizeof(ret)) == 0)
+		return -1;
+
 	if ((fd = open("/dev/null", O_RDWR)) != -1) {
 		dup2(fd, 0);
 		dup2(fd, 1);
 		dup2(fd, 2);
 	}
+
 	logger(10, 0, "Starting init");
 	execve("/sbin/init", argv, envp);
 	execve("/etc/init", argv, envp);
 	execve("/bin/init", argv, envp);
-
 	ret = VZ_FS_BAD_TMPL;
-	write(err_p, &ret, sizeof(ret));
-env_err:
+	write(arg->err_p, &ret, sizeof(ret));
 	return ret;
 }
 
 static int vz_real_env_create(vps_handler *h, envid_t veid, vps_res *res,
 	int wait_p, int old_wait_p, int err_p, env_create_FN fn, void *data)
-{
-	int ret, pid;
 
-	if ((ret = vz_chroot(res->fs.root)))
-		return ret;
-	if ((ret = vz_setluid(veid)))
-		return ret;
-	if ((ret = set_ublimit(h, veid, &res->ub)))
-		return ret;
-	/* Create another process for proper resource accounting */
-	if ((pid = fork()) < 0) {
-		logger(-1, errno, "Unable to fork");
-		return VZ_RESOURCE_ERROR;
-	} else if (pid == 0) {
-		if ((ret = vps_set_cap(veid, &res->env, &res->cap)))
-			goto env_err;
-		if (fn == NULL) {
-			ret = _env_create(h, veid, wait_p, err_p, (void *)res);
-		} else {
-			ret = fn(h, veid, wait_p, old_wait_p, err_p, data);
-		}
-env_err:
-		if (ret)
-			write(STDIN_FILENO, &ret, sizeof(ret));
-		exit(ret);
-	}
-	return 0;
+{
+	struct arg_start arg;
+
+	arg.res = res;
+	arg.wait_p = wait_p;
+	arg.old_wait_p = old_wait_p;
+	arg.err_p = err_p;
+	arg.veid = veid;
+	arg.h = h;
+	arg.data = data;
+	arg.fn = fn;
+
+	return h->env_create(&arg);
 }
 
 #define MAX_OSREL_LEN 128
@@ -708,10 +544,12 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 		logger(-1, 0, "Container is already running");
 		return VZ_VE_RUNNING;
 	}
-	if ((ret = check_ub(&res->ub)))
+	if ((ret = check_ub(h, &res->ub)))
 		return ret;
 
 	ploop = ve_private_is_ploop(res->fs.private);
+	if (ploop && !is_ploop_supported())
+		return VZ_PLOOP_UNSUP;
 
 	dist_name = get_dist_name(&res->tmpl);
 	ret = read_dist_actions(dist_name, DIST_DIR, &actions);
@@ -720,12 +558,14 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 		return ret;
 	logger(0, 0, "Starting container ...");
 	/* if CT is mounted -- umount first, to cleanup mount state */
-	if (vps_is_mounted(res->fs.root)) {
+	if (vps_is_mounted(res->fs.root, res->fs.private)) {
 		vps_umount(h, veid, &res->fs, skip);
 	}
+#ifdef HAVE_PLOOP
 	else if (ploop && (is_image_mounted(res->fs.private)))
 		vzctl_umount_image(res->fs.private);
-	if (!vps_is_mounted(res->fs.root)) {
+#endif
+	if (!vps_is_mounted(res->fs.root, res->fs.private)) {
 		/* increase quota to perform setup */
 		if (!ploop)
 			quota_inc(&res->dq, 100);
@@ -838,7 +678,7 @@ err:
 		/* restore original quota values */
 		if (!ploop)
 			vps_set_quota(veid, &res->dq);
-		if (vps_is_mounted(res->fs.root))
+		if (vps_is_mounted(res->fs.root, res->fs.private))
 			vps_umount(h, veid, &res->fs, skip);
 	}
 	close(wait_p[0]);
@@ -869,29 +709,31 @@ static int real_env_stop(vps_handler *h, envid_t veid, const char *vps_root,
 {
 	int ret;
 
-	if ((ret = vz_chroot(vps_root)))
-		return ret;
-	if ((ret = vz_setluid(veid)))
+	if ((ret = h->setcontext(veid)))
 		return ret;
 	close_fds(1, h->vzfd, -1);
-	if ((ret = vz_env_create_ioctl(h, veid, VE_ENTER)) < 0) {
-		if (errno == ESRCH)
-			return 0;
+	ret = h->enter(h, veid, vps_root, 0);
+	if (ret == VZ_VE_NOT_RUNNING)
+		/* Ignore "VE not running" error here */
+		return 0;
+	else if (ret) {
 		logger(-1, errno, "Container enter failed");
 		return ret;
 	}
-	close(h->vzfd);
+
 	switch (stop_mode) {
 		case M_REBOOT:
 		{
 			char *argv[] = {"reboot", NULL};
 			execvep(argv[0], argv, NULL);
+			return VZ_STOP_ERROR;
 			break;
 		}
 		case M_HALT:
 		{
 			char *argv[] = {"halt", NULL};
 			execvep(argv[0], argv, NULL);
+			return VZ_STOP_ERROR;
 			break;
 		}
 		case M_KILL:
@@ -905,21 +747,49 @@ static int real_env_stop(vps_handler *h, envid_t veid, const char *vps_root,
 	return 0;
 }
 
+static int wait_child(int pid, int ignore_kill)
+{
+	int status, ret;
+
+	while ((ret = waitpid(pid, &status, 0)) == -1)
+		if (errno != EINTR)
+			break;
+
+	if (ret < 0) {
+		logger(-1, errno, "Error in waitpid(%d)", pid);
+		return VZ_SYSTEM_ERROR;
+	}
+
+	ret = 0;
+	if (WIFEXITED(status) && (ret = WEXITSTATUS(status)))
+		logger(-1, 0, "Child %d exited with status %d", pid, ret);
+	else if (!ignore_kill && WIFSIGNALED(status)) {
+		logger(-1, 0, "Child %d terminated with signal %d",
+				pid, WTERMSIG(status));
+		ret = VZ_SYSTEM_ERROR;
+	}
+
+	return ret;
+}
+
 static int env_stop(vps_handler *h, envid_t veid, const char *root,
 		int stop_mode)
 {
-	struct sigaction act, actold;
-	int i, pid, ret = 0;
+	int i, pid, ret, tout = 0;
 
-	sigaction(SIGCHLD, NULL, &actold);
-	sigemptyset(&act.sa_mask);
-	act.sa_handler = SIG_IGN;
-	act.sa_flags = SA_NOCLDSTOP;
-	sigaction(SIGCHLD, &act, NULL);
-
-	logger(0, 0, "Stopping container ...");
 	if (stop_mode == M_KILL)
 		goto kill_vps;
+
+	if (!is_vz_kernel(h)) {
+		logger(-1, 0, "Due to lack of proper support in this kernel, "
+		"container can't be cleanly\n"
+		"stopped from the host system. Please stop it from inside, "
+		"or use --fast option\n"
+		"to forcibly kill it (note it is unsafe operation).");
+		ret = VZ_BAD_KERNEL;
+		goto out;
+	}
+	logger(0, 0, "Stopping container ...");
 	if ((pid = fork()) < 0) {
 		logger(-1, errno, "Can not fork");
 		ret = VZ_RESOURCE_ERROR;
@@ -928,6 +798,9 @@ static int env_stop(vps_handler *h, envid_t veid, const char *root,
 		ret = real_env_stop(h, veid, root, stop_mode);
 		exit(ret);
 	}
+	if (wait_child(pid, 0)) /* reboot/halt failed, retry with kill */
+		goto kill_vps;
+
 	for (i = 0; i < MAX_SHTD_TM; i++) {
 		sleep(1);
 		if (!vps_is_run(h, veid)) {
@@ -937,15 +810,25 @@ static int env_stop(vps_handler *h, envid_t veid, const char *root,
 	}
 
 kill_vps:
+	logger(0, 0, "Killing container ...");
+	ret = h->destroy(h, veid);
+	if (!is_vz_kernel(h))
+		goto wait;
+
 	if ((pid = fork()) < 0) {
 		ret = VZ_RESOURCE_ERROR;
 		logger(-1, errno, "Can not fork");
-		goto err;
+		goto out;
 
 	} else if (pid == 0) {
 		ret = real_env_stop(h, veid, root, M_KILL);
 		exit(ret);
 	}
+	ret = wait_child(pid, 1);
+	if (ret)
+		goto out;
+
+wait:
 	ret = VZ_STOP_ERROR;
 	for (i = 0; i < MAX_SHTD_TM; i++) {
 		usleep(500000);
@@ -954,13 +837,14 @@ kill_vps:
 			break;
 		}
 	}
+	tout = 1;
 out:
 	if (ret)
-		logger(-1, 0, "Unable to stop container: operation timed out");
+		logger(-1, 0, "Unable to stop container%s",
+				tout ? ": operation timed out" : "");
 	else
 		logger(0, 0, "Container was stopped");
-err:
-	sigaction(SIGCHLD, &actold, NULL);
+
 	return ret;
 }
 
@@ -998,14 +882,21 @@ int vps_stop(vps_handler *h, envid_t veid, vps_param *param, int stop_mode,
 			}
 		}
 	}
+
 	/* get CT IP addresses for cleanup */
-	get_vps_ip(h, veid, &param->del_res.net.ip);
+	if (is_vz_kernel(h))
+		get_vps_ip(h, veid, &param->del_res.net.ip);
+
 	if ((ret = env_stop(h, veid, res->fs.root, stop_mode)))
 		goto end;
+
 	mod_cleanup(h, veid, action, param);
+
 	/* Cleanup CT IPs */
-	run_net_script(veid, DEL, &param->del_res.net.ip,
+	if (is_vz_kernel(h))
+		run_net_script(veid, DEL, &param->del_res.net.ip,
 			STATE_STOPPING, param->res.net.skip_arpdetect);
+
 	if (!(skip & SKIP_UMOUNT))
 		ret = vps_umount(h, veid, &res->fs, skip);
 
