@@ -23,8 +23,10 @@
 #include "image.h"
 #include "snapshot.h"
 #include "vzerror.h"
+#include "vzconfig.h"
 
-#define VZCTL_VE_DUMP_DIR     "/dump"
+#define VZCTL_VE_DUMP_DIR	"/dump"
+#define VZCTL_VE_CONF		"ve.conf"
 
 #define SNAPSHOT_XML	"Snapshots.xml"
 #define GET_SNAPSHOT_XML(buf, ve_private) \
@@ -40,6 +42,13 @@ static void vzctl_get_snapshot_dumpfile(const char *private, const char *guid,
 	snprintf(buf, len, "%s" VZCTL_VE_DUMP_DIR "/%s", private, guid);
 }
 
+static void vzctl_get_snapshot_ve_conf(const char *private, const char *guid,
+		char *buf, int len)
+{
+	snprintf(buf, len, "%s" VZCTL_VE_DUMP_DIR "/%s." VZCTL_VE_CONF,
+			private, guid);
+}
+
 static int is_snapshot_supported(const char *ve_private)
 {
 	if (! ve_private_is_ploop(ve_private)) {
@@ -47,7 +56,8 @@ static int is_snapshot_supported(const char *ve_private)
 				"for ploop-based CTs");
 		return 0;
 	}
-	return 1;
+
+	return is_ploop_supported();
 }
 
 int vzctl_env_create_snapshot(vps_handler *h, envid_t veid,
@@ -59,6 +69,7 @@ int vzctl_env_create_snapshot(vps_handler *h, envid_t veid,
 	char guid[39];
 	char fname[PATH_MAX];
 	char tmp[PATH_MAX];
+	char snap_ve_conf[PATH_MAX];
 	struct ploop_snapshot_param image_param = {};
 	struct ploop_merge_param merge_param = {};
 	struct ploop_disk_images_data *di = NULL;
@@ -69,11 +80,8 @@ int vzctl_env_create_snapshot(vps_handler *h, envid_t veid,
 	tree = vzctl_alloc_snapshot_tree();
 	if (tree == NULL)
 		return VZ_RESOURCE_ERROR;
-	di = ploop_alloc_diskdescriptor();
-	if (di == NULL)
-		goto err;
 	if (param->guid == NULL) {
-		if (ploop_uuid_generate(guid, sizeof(guid)))
+		if (ploop.uuid_generate(guid, sizeof(guid)))
 			goto err;
 	} else
 		snprintf(guid, sizeof(guid), "%s", param->guid);
@@ -87,7 +95,7 @@ int vzctl_env_create_snapshot(vps_handler *h, envid_t veid,
 	}
 	logger(0, 0, "Creating snapshot %s", guid);
 	GET_DISK_DESCRIPTOR(fname, fs->private);
-	if (ploop_read_diskdescriptor(fname, di)) {
+	if (ploop.read_disk_descr(&di, fname)) {
 		logger(-1, 0, "Failed to read %s", fname);
 		goto err;
 	}
@@ -101,6 +109,14 @@ int vzctl_env_create_snapshot(vps_handler *h, envid_t veid,
 		logger(-1, 0, "Failed to store %s", tmp);
 		goto err;
 	}
+	// Store ve.conf
+	get_vps_conf_path(veid, fname, sizeof(fname));
+	vzctl_get_snapshot_ve_conf(fs->private, guid,
+			snap_ve_conf, sizeof(snap_ve_conf));
+	make_dir(snap_ve_conf, 0);
+	if (cp_file(snap_ve_conf, tmp))
+		goto err1;
+
 	if (!(param->flags & SNAPSHOT_SKIP_SUSPEND))
 		run = vps_is_run(h, veid);
 	/* 1 freeze */
@@ -111,11 +127,10 @@ int vzctl_env_create_snapshot(vps_handler *h, envid_t veid,
 	}
 	/* 2 create snapshot with specified guid */
 	image_param.guid = guid;
-	image_param.snap_guid = 1;
-	ret = ploop_create_snapshot(di, &image_param);
+	ret = ploop.create_snapshot(di, &image_param);
 	if (ret) {
 		logger(-1, 0, "Failed to create snapshot: %s [%d]",
-				ploop_get_last_error(), ret);
+				ploop.get_last_error(), ret);
 		goto err1;
 	}
 	/* 3 store dump & continue */
@@ -134,24 +149,25 @@ int vzctl_env_create_snapshot(vps_handler *h, envid_t veid,
 		logger(-1, 0, "Failed to rename %s -> %s", tmp, fname);
 	logger(0, 0, "Snapshot %s has been successfully created",
 			guid);
-	ploop_free_diskdescriptor(di);
+	ploop.free_diskdescriptor(di);
 	return 0;
 err2:
 	// merge top_delta
-	ret = ploop_merge_snapshot(di, &merge_param);
+	ret = ploop.merge_snapshot(di, &merge_param);
 	if (ret)
 		logger(-1, 0, "Rollback failed, ploop_merge_snapshot %s: %s [%d]",
-				guid, ploop_get_last_error(), ret);
+				guid, ploop.get_last_error(), ret);
 
 err1:
 	if (run)
 		cpt_cmd(h, veid, fs->root, CMD_CHKPNT, CMD_RESUME, 0);
 	unlink(tmp);
+	unlink(snap_ve_conf);
 
 err:
 	logger(-1, 0, "Failed to create snapshot");
 	if (di != NULL)
-		ploop_free_diskdescriptor(di);
+		ploop.free_diskdescriptor(di);
 	if (tree != NULL)
 		vzctl_free_snapshot_tree(tree);
 	return VZCTL_E_CREATE_SNAPSHOT;
@@ -161,10 +177,13 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 		vps_param *g_p,	const fs_param *fs, const char *guid)
 {
 	int ret, run;
+	int flags = 0;
 	cpt_param cpt = {};
 	char fname[PATH_MAX];
-	char tmp[PATH_MAX];
-	char prev_top_guid[39];
+	char snap_xml_tmp[PATH_MAX];
+	char ve_conf_tmp[PATH_MAX] = "";
+	char topdelta_fname[PATH_MAX] = "";
+	char dd_tmp[PATH_MAX] = "";
 	char dumpfile[PATH_MAX];
 	struct vzctl_snapshot_tree *tree = NULL;
 	struct ploop_disk_images_data *di = NULL;
@@ -179,9 +198,6 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 	tree = vzctl_alloc_snapshot_tree();
 	if (tree == NULL)
 		goto err;
-	di = ploop_alloc_diskdescriptor();
-	if (di == NULL)
-		goto err;
 
 	ret = vzctl_read_snapshot_tree(fname, tree);
 	if (ret) {
@@ -193,17 +209,17 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 		goto err;
 	}
 	GET_DISK_DESCRIPTOR(fname, fs->private);
-	ret = ploop_read_diskdescriptor(fname, di);
+	ret = ploop.read_disk_descr(&di, fname);
 	if (ret) {
 		logger(-1, 0, "Failed to read %s", fname);
 		goto err;
 	}
 	logger(0, 0, "Switching to snapshot %s", guid);
 	vzctl_snapshot_tree_set_current(tree, guid);
-	GET_SNAPSHOT_XML_TMP(tmp, fs->private);
-	ret = vzctl_store_snapshot_tree(tmp, tree);
+	GET_SNAPSHOT_XML_TMP(snap_xml_tmp, fs->private);
+	ret = vzctl_store_snapshot_tree(snap_xml_tmp, tree);
 	if (ret) {
-		logger(-1, 0, "Failed to store %s", tmp);
+		logger(-1, 0, "Failed to store %s", snap_xml_tmp);
 		goto err;
 	}
 
@@ -212,18 +228,37 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 		goto err1;
 	/* freeze */
 	if (run) {
-		snprintf(prev_top_guid, sizeof(prev_top_guid), "%s", di->top_guid);
+		/* do not destroy current top delta */
+		flags = PLOOP_SNAP_SKIP_TOPDELTA_DESTROY;
+		if (ploop.get_top_delta_fname(di, topdelta_fname, sizeof(topdelta_fname)))
+			goto err1;
 		ret = vps_chkpnt(h, veid, fs, CMD_SUSPEND, &cpt);
 		if (ret)
 			goto err1;
-	} else if (vps_is_mounted(fs->root)) {
+	} else if (vps_is_mounted(fs->root, fs->private)) {
 		if (vps_umount(h, veid, fs, 0))
 			goto err1;
 	}
 	/* switch snapshot */
-	ret = ploop_switch_snapshot(di, guid, (run ? PLOOP_LEAVE_TOP_DELTA : 0));
-	if (ret)
+	/* preserve DiskDescriptor.xml -> DiskDescriptor.xml.orig */
+	snprintf(dd_tmp, sizeof(dd_tmp), "%s.orig", fname);
+	if (cp_file(dd_tmp, fname))
+		goto err1;
+
+	ret = ploop.switch_snapshot(di, guid, flags);
+	if (ret) {
+		unlink(dd_tmp);
 		goto err2;
+	}
+	/* restore ve.conf */
+	vzctl_get_snapshot_ve_conf(fs->private, guid, fname, sizeof(fname));
+	if (stat_file(fname)) {
+		get_vps_conf_path(veid, fname, sizeof(fname) - 4);
+		strcat(fname, ".tmp");
+		if (cp_file(ve_conf_tmp, fname))
+			goto err2;
+	}
+
 	/* stop CT */
 	if (run) {
 		ret = cpt_cmd(h, veid, fs->root, CMD_CHKPNT, CMD_KILL, 0);
@@ -231,45 +266,66 @@ int vzctl_env_switch_snapshot(vps_handler *h, envid_t veid,
 			goto err3;
 		if (vps_umount(h, veid, fs, 0))
 			goto err3;
-		if (ploop_delete_snapshot(di, prev_top_guid))
-			logger(-1, 0, "Failed to remove old top guid %s",
-				prev_top_guid);
+	}
+	if (ve_conf_tmp[0] != '\0') {
+		get_vps_conf_path(veid, fname, sizeof(fname));
+		if (rename(ve_conf_tmp, fname))
+			logger(-1, errno, "Failed to rename %s -> %s",
+					ve_conf_tmp, fname);
 	}
 	/* resume CT in case dump file exists (no rollback, ignore error) */
 	vzctl_get_snapshot_dumpfile(fs->private, guid, dumpfile, sizeof(dumpfile));
 	if (stat_file(dumpfile)) {
+		vps_param *param;
+
+		param = reread_vps_config(veid);
 		cpt.dumpfile = dumpfile;
-		if (vps_restore(h, veid, g_p, CMD_RESTORE, &cpt))
+		if (vps_restore(h, veid, (param) ? param : g_p,
+					CMD_RESTORE, &cpt))
 			logger(-1, 0, "Failed to resume Container");
+		free_vps_param(param);
 	}
 	GET_SNAPSHOT_XML(fname, fs->private);
-	if (rename(tmp, fname))
-		logger(-1, 0, "Failed to rename %s %s", tmp, fname);
+	if (rename(snap_xml_tmp, fname))
+		logger(-1, 0, "Failed to rename %s %s", snap_xml_tmp, fname);
+
+	/* cleanup */
+	if (dd_tmp[0] != '\0' && unlink(dd_tmp))
+		logger(-1, errno, "Failed to unlink %s", dd_tmp);
+	if (topdelta_fname[0] != '\0' && unlink(topdelta_fname))
+		logger(-1, errno, "Failed to unlink %s", topdelta_fname);
 
 	logger(0, 0, "Container has been successfully switched "
 			"to another snapshot");
-	ploop_free_diskdescriptor(di);
+	ploop.free_diskdescriptor(di);
+
 	return 0;
 
 err3:
-	/* FIXME rollback ploop_switch_snapshot:
-	   ploop_di_remove_image(di, di->top_guid, &fname);
-	   free(di->top_guid);
-	   di->top_guid = strdup(prev_top_guid);
-	   ploop_store_diskdescriptor(di);
-	   unlink(fname);
-	   */
+	/* Restore original DiskDescriptor.xml */
+	GET_DISK_DESCRIPTOR(fname, fs->private);
+	if (rename(dd_tmp, fname)) {
+		logger(-1, errno, "Failed to rename %s -> %s",
+				dd_tmp, fname);
+		goto err2;
+	}
+	/* remove new top delta */
+	if (ploop.get_top_delta_fname(di, topdelta_fname, sizeof(topdelta_fname)) == 0)
+		if (unlink(topdelta_fname))
+			logger(-1, errno, "Failed to unlink %s", topdelta_fname);
+
 err2:
 	if (cpt_cmd(h, veid, fs->root, CMD_CHKPNT, CMD_RESUME, 0))
 		logger(-1, 0, "Failed to resume Container on error");
 err1:
-	unlink(tmp);
+	unlink(snap_xml_tmp);
 err:
 	logger(-1, 0, "Failed to switch to snapshot %s", guid);
 	if (tree != NULL)
 		vzctl_free_snapshot_tree(tree);
 	if (di != NULL)
-		ploop_free_diskdescriptor(di);
+		ploop.free_diskdescriptor(di);
+
 	return VZCTL_E_SWITCH_SNAPSHOT;
 }
 
@@ -318,10 +374,15 @@ int vzctl_env_delete_snapshot(vps_handler *h, envid_t veid,
 	if (stat_file(fname)) {
 		logger(1, 0, "Deleting CT dump %s", fname);
 		if (unlink(fname))
-			logger(-1, 0, "Failed to delete dump %s",
+			logger(-1, errno, "Failed to delete dump %s",
 					fname);
 	}
 
+	// delete ve.conf
+	vzctl_get_snapshot_ve_conf(fs->private, guid, fname, sizeof(fname));
+	if (stat_file(fname) && unlink(fname))
+		logger(-1, errno, "Failed to delete %s", fname);
+
 	// move snapshot.xml to its place
 	GET_SNAPSHOT_XML(fname, fs->private);
 	if (rename(tmp, fname))
