@@ -18,6 +18,7 @@
 
 #include <stdlib.h>
 #include <sys/types.h>
+#include <sys/mount.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <string.h>
@@ -25,13 +26,14 @@
 #include <linux/vzcalluser.h>
 #include <errno.h>
 #include <signal.h>
+#include <limits.h>
 
 #include "vzctl.h"
 #include "vzctl_param.h"
 #include "env.h"
 #include "logger.h"
 #include "exec.h"
-#include "config.h"
+#include "vzconfig.h"
 #include "vzerror.h"
 #include "create.h"
 #include "destroy.h"
@@ -102,7 +104,8 @@ static struct option set_opt[] = {
 	/*	fs param */
 	{"root",	required_argument, NULL, PARAM_ROOT},
 	{"private",	required_argument, NULL, PARAM_PRIVATE},
-	{"noatime",	required_argument, NULL, PARAM_NOATIME},
+	{"noatime",	required_argument, NULL, PARAM_IGNORED},
+	{"mount_opts",	required_argument, NULL, PARAM_MOUNT_OPTS},
 	/*	template	*/
 	{"ostemplate",	required_argument, NULL, PARAM_OSTEMPLATE},
 	/*	Cpu	*/
@@ -211,7 +214,7 @@ static int parse_opt(envid_t veid, int argc, char *argv[], struct option *opt,
 static int parse_startstop_opt(int argc, char **argv, vps_param *param,
 		int start, int stop)
 {
-	int c, ret = 0;
+	int c, ret = 0, idx;
 	struct option start_options[] = {
 		{"force", no_argument, NULL, PARAM_FORCE},
 		{"skip_ve_setup", no_argument, NULL, PARAM_SKIP_VE_SETUP},
@@ -222,7 +225,7 @@ static int parse_startstop_opt(int argc, char **argv, vps_param *param,
 	};
 
 	while (1) {
-		c = getopt_long (argc, argv, "", start_options, NULL);
+		c = getopt_long (argc, argv, "", start_options, &idx);
 		if (c == -1)
 			break;
 		switch (c) {
@@ -230,38 +233,46 @@ static int parse_startstop_opt(int argc, char **argv, vps_param *param,
 			if (start)
 				param->opt.start_force = YES;
 			else
-				ret = VZ_INVALID_PARAMETER_SYNTAX;
+				goto err;
 			break;
 		case PARAM_SKIP_VE_SETUP:
 			if (start)
 				param->opt.skip_setup = YES;
 			else
-				ret = VZ_INVALID_PARAMETER_SYNTAX;
+				goto err;
 			break;
 		case PARAM_WAIT:
 			if (start)
 				param->res.misc.wait = YES;
 			else
-				ret = VZ_INVALID_PARAMETER_SYNTAX;
+				goto err;
 			break;
 		case PARAM_FAST:
 			if (stop)
 				param->opt.fast_kill = YES;
 			else
-				ret = VZ_INVALID_PARAMETER_SYNTAX;
+				goto err;
 			break;
 		case PARAM_SKIP_UMOUNT:
 			if (stop)
 				param->opt.skip_umount = YES;
 			else
-				ret = VZ_INVALID_PARAMETER_SYNTAX;
+				goto err;
 			break;
 		default:
 			ret = VZ_INVALID_PARAMETER_SYNTAX;
 			break;
 		}
 	}
+
 	return ret;
+
+err:
+	fprintf(stderr, "Option --%s is not applicable to %s command\n",
+			start_options[idx].name,
+			start ? "start" : "stop");
+
+	return VZ_INVALID_PARAMETER_SYNTAX;
 }
 
 static int start(vps_handler *h, envid_t veid, vps_param *g_p,
@@ -371,6 +382,7 @@ static int destroy(vps_handler *h, envid_t veid, vps_param *g_p,
 	return ret;
 }
 
+#ifdef HAVE_PLOOP
 static int parse_convert_opt(envid_t veid, int argc, char **argv,
 	vps_param *param)
 {
@@ -414,6 +426,58 @@ static int convert(vps_handler *h, envid_t veid, vps_param *g_p,
 			&g_p->res.fs, &g_p->res.dq, mode);
 }
 
+static int compact(vps_handler *h, envid_t veid, vps_param *g_p,
+		vps_param *cmd_p)
+{
+	char *cmd;
+	int mounted;
+	int ret;
+	const char *private	= g_p->res.fs.private;
+	const char *root	= g_p->res.fs.root;
+	char fname[PATH_MAX];
+
+	if (check_var(root, "VE_ROOT is not set"))
+		return VZ_VE_ROOT_NOTSET;
+
+	if (check_var(private, "VE_PRIVATE is not set"))
+		return VZ_VE_PRIVATE_NOTSET;
+
+	if (!stat_file(private)) {
+		logger(-1, 0, "Container private area %s does not exist",
+				private);
+		return VZ_FS_NOPRVT;
+	}
+
+	if (!ve_private_is_ploop(private)) {
+		logger(0, 0, "Compact only makes sense for ploop containers");
+		return 0;
+	}
+
+	mounted = vps_is_mounted(root, private);
+	if (!mounted)
+	{
+		ret = vps_mount(h, veid, &g_p->res.fs, &g_p->res.dq,
+				SKIP_ACTION_SCRIPT);
+		if (ret != 0)
+			return ret;
+	}
+
+	GET_DISK_DESCRIPTOR(fname, private);
+	if (asprintf(&cmd, "ploop balloon discard %s", fname) < 0) {
+		logger(-1, ENOMEM, "Can't allocate string for cmd");
+		return VZ_RESOURCE_ERROR;
+	}
+	logger(1, 0, "Executing %s", cmd);
+	ret = system(cmd);
+	free(cmd);
+
+	if (!mounted)
+		vps_umount(h, veid, &g_p->res.fs, SKIP_ACTION_SCRIPT);
+
+	return (ret == 0) ? 0 : VZCTL_E_COMPACT_IMAGE;
+}
+#endif /* HAVE_PLOOP */
+
 static int parse_chkpnt_opt(int argc, char **argv, vps_param *vps_p)
 {
 	int c;
@@ -559,6 +623,7 @@ err_syntax:
 	return VZ_INVALID_PARAMETER_SYNTAX;
 }
 
+#ifdef HAVE_PLOOP
 static int parse_snapshot_create_opt(envid_t veid, int argc, char **argv,
 		vps_param *param)
 {
@@ -607,6 +672,7 @@ static int parse_snapshot_delete_opt(envid_t veid, int argc, char **argv,
 
 	return ret;
 }
+#endif /* HAVE_PLOOP */
 
 static int check_set_ugidlimit(unsigned long *cur, unsigned long *old,
 		int loud)
@@ -686,6 +752,14 @@ static int check_set_mode(vps_handler *h, envid_t veid, int setmode, int apply,
 					"on a running container");
 		found++;
 	}
+	/* Changing mount_opts */
+	if ( new_res->fs.mount_opts && strcmp(new_res->fs.mount_opts,
+				old_res->fs.mount_opts)) {
+		if (loud)
+			logger(-1, 0, "Unable to change mount options "
+					"on a running container");
+		found++;
+	}
 
 	if (!found)
 		return 0;
@@ -911,12 +985,23 @@ static int set(vps_handler *h, envid_t veid, vps_param *g_p, vps_param *vps_p,
 		}
 	}
 	/* Resize ploop image if --diskspace is supplied */
-	if (ve_private_is_ploop(g_p->res.fs.private) &&
-			cmd_p->res.dq.diskspace)
+	if (cmd_p->res.dq.diskspace &&
+			ve_private_is_ploop(g_p->res.fs.private))
 	{
+		if (! is_ploop_supported())
+			return VZ_PLOOP_UNSUP;
+#ifdef HAVE_PLOOP
 		if ((ret = vzctl_resize_image(g_p->res.fs.private,
 						cmd_p->res.dq.diskspace[1])))
 			return ret;
+#endif
+	}
+	/* Warn that --diskinodes is ignored for ploop CT */
+	if (cmd_p->res.dq.diskinodes &&
+			ve_private_is_ploop(g_p->res.fs.private))
+	{
+		logger(0, 0, "Warning: --diskinodes is ignored for "
+				"ploop-based container");
 	}
 	/* Skip applying parameters on stopped CT */
 	if (cmd_p->opt.save && !is_run) {
@@ -947,18 +1032,6 @@ err:
 	return ret;
 }
 
-static int mount(vps_handler *h, envid_t veid, vps_param *g_p,
-	vps_param *cmd_p)
-{
-	return vps_mount(h, veid, &g_p->res.fs, &g_p->res.dq, 0);
-}
-
-static int umount(vps_handler *h, envid_t veid, vps_param *g_p,
-	vps_param *cmd_p)
-{
-	return vps_umount(h, veid, &g_p->res.fs, 0);
-}
-
 static int enter(vps_handler *h, envid_t veid, const char *root,
 		int argc, char **argv)
 {
@@ -1095,9 +1168,9 @@ static int show_status(vps_handler *h, envid_t veid, vps_param *param)
 	get_vps_conf_path(veid, buf, sizeof(buf));
 	if (fs->private != NULL && stat_file(fs->private) && stat_file(buf))
 		exist = 1;
-	mounted = vps_is_mounted(fs->root);
+	mounted = (vps_is_mounted(fs->root, fs->private) == 1);
 	run = vps_is_run(h, veid);
-	if (exist == 1) {
+	if (exist && !run) {
 		get_dump_file(veid, param->res.cpt.dumpdir, buf, sizeof(buf));
 		if (stat_file(buf))
 			suspended = 1;
@@ -1137,9 +1210,11 @@ int parse_action_opt(envid_t veid, act_t action, int argc, char *argv[],
 	case ACTION_CREATE:
 		ret = parse_create_opt(veid, argc, argv, param);
 		break;
+#ifdef HAVE_PLOOP
 	case ACTION_CONVERT:
 		ret = parse_convert_opt(veid, argc, argv, param);
 		break;
+#endif
 	case ACTION_START:
 		ret = parse_startstop_opt(argc, argv, param, 1, 0);
 		break;
@@ -1165,10 +1240,6 @@ int parse_action_opt(envid_t veid, act_t action, int argc, char *argv[],
 			}
 		}
 		break;
-	case ACTION_DESTROY:
-		break;
-	case ACTION_MOUNT:
-		break;
 	case ACTION_EXEC:
 	case ACTION_EXEC2:
 	case ACTION_EXEC3:
@@ -1194,6 +1265,7 @@ int parse_action_opt(envid_t veid, act_t action, int argc, char *argv[],
 		break;
 	case ACTION_CONSOLE:
 		break;
+#ifdef HAVE_PLOOP
 	case ACTION_SNAPSHOT_CREATE:
 		ret = parse_snapshot_create_opt(veid, argc, argv, param);
 		break;
@@ -1201,8 +1273,7 @@ int parse_action_opt(envid_t veid, act_t action, int argc, char *argv[],
 	case ACTION_SNAPSHOT_SWITCH:
 		ret = parse_snapshot_delete_opt(veid, argc, argv, param);
 		break;
-	case ACTION_SNAPSHOT_LIST:
-		break;
+#endif
 	default :
 		if ((argc - 1) > 0) {
 			fprintf (stderr, "Invalid options: ");
@@ -1234,6 +1305,14 @@ int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 			return VZ_BAD_KERNEL;
 		}
 	}
+
+	if (!is_vz_kernel(h)) {
+		/* No simfs support, emulate it with a bind mount */
+		g_p->res.fs.flags |= MS_BIND;
+		/* No quotas, regardless of what the config says */
+		g_p->res.dq.enable = NO;
+	}
+
 	if (action != ACTION_EXEC &&
 		action != ACTION_EXEC2 &&
 		action != ACTION_EXEC3 &&
@@ -1266,10 +1345,10 @@ int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 		ret = destroy(h, veid, g_p, cmd_p);
 		break;
 	case ACTION_MOUNT:
-		ret = mount(h, veid, g_p, cmd_p);
+		ret = vps_mount(h, veid, &g_p->res.fs, &g_p->res.dq, 0);
 		break;
 	case ACTION_UMOUNT:
-		ret = umount(h, veid, g_p, cmd_p);
+		ret = vps_umount(h, veid, &g_p->res.fs, 0);
 		break;
 	case ACTION_START:
 		ret = start(h, veid, g_p, cmd_p);
@@ -1280,9 +1359,14 @@ int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 	case ACTION_RESTART:
 		ret = restart(h, veid, g_p, cmd_p);
 		break;
+#ifdef HAVE_PLOOP
 	case ACTION_CONVERT:
 		ret = convert(h, veid, g_p, cmd_p);
 		break;
+	case ACTION_COMPACT:
+		ret = compact(h, veid, g_p, cmd_p);
+		break;
+#endif
 	case ACTION_SET:
 		if (veid == 0)
 			ret = set_ve0(h, g_p, vps_p, cmd_p);
@@ -1364,6 +1448,8 @@ int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 		ret = quota_init(veid, g_p->res.fs.private, &g_p->res.dq);
 		break;
 #undef CHECK_DQ
+
+#ifdef HAVE_PLOOP
 	case ACTION_SNAPSHOT_CREATE:
 		ret = vzctl_env_create_snapshot(h, veid,
 				&g_p->res.fs, &cmd_p->snap);
@@ -1379,6 +1465,7 @@ int run_action(envid_t veid, act_t action, vps_param *g_p, vps_param *vps_p,
 	case ACTION_SNAPSHOT_LIST:
 		ret = vzctl_env_list_snapshot_tree(g_p->res.fs.private);
 		break;
+#endif
 	case ACTION_CUSTOM:
 		ret = mod_setup(h, veid, 0, 0, &g_action, g_p);
 		break;
