@@ -28,7 +28,7 @@
 #include "env.h"
 #include "logger.h"
 #include "exec.h"
-#include "config.h"
+#include "vzconfig.h"
 #include "vzerror.h"
 #include "types.h"
 #include "util.h"
@@ -67,7 +67,14 @@ static void usage(int rc)
 "   [--diskspace <kbytes>] [--private <path>] [--root <path>]\n"
 "vzctl start <ctid> [--force] [--wait]\n"
 "vzctl destroy | mount | umount | stop | restart | status <ctid>\n"
+#ifdef HAVE_PLOOP
 "vzctl convert <ctid> [--layout ploop[:mode]] [--diskspace <kbytes>]\n"
+"vzctl compact <ctid>\n"
+"vzctl snapshot <ctid> [--id <uuid>] [--name <name>] [--description <desc>]\n"
+"   [--skip-suspend]\n"
+"vzctl snapshot-switch | snapshot-delete <ctid> --id <uuid>\n"
+"vzctl snapshot-list\n"
+#endif
 "vzctl quotaon | quotaoff | quotainit <ctid>\n"
 "vzctl console <ctid> [ttyno]\n"
 "vzctl enter <ctid> [--exec <command> [arg ...]]\n"
@@ -83,8 +90,8 @@ static void usage(int rc)
 "   [--userpasswd <user>:<passwd>]\n"
 "   [--cpuunits <N>] [--cpulimit <N>] [--cpus <N>] [--cpumask <cpus>]\n"
 "   [--diskspace <soft>[:<hard>]] [--diskinodes <soft>[:<hard>]]\n"
-"   [--quotatime <N>] [--quotaugidlimit <N>]\n"
-"   [--noatime yes|no] [--capability <name>:on|off ...]\n"
+"   [--quotatime <N>] [--quotaugidlimit <N>] [--mount_opts <opt>[,<opt>...]]\n"
+"   [--capability <name>:on|off ...]\n"
 "   [--devices b|c:major:minor|all:r|w|rw]\n"
 "   [--devnodes device:r|w|rw|none]\n"
 "   [--netif_add <ifname[,mac,host_ifname,host_mac,bridge]]>]\n"
@@ -193,8 +200,12 @@ int main(int argc, char *argv[], char *envp[])
 		action = ACTION_ENTER;
 	} else if (!strcmp(argv[1], "console")) {
 		action = ACTION_CONSOLE;
+#ifdef HAVE_PLOOP
 	} else if (!strcmp(argv[1], "convert")) {
 		action = ACTION_CONVERT;
+	} else if (!strcmp(argv[1], "compact")) {
+		action = ACTION_COMPACT;
+#endif
 	} else if (!strcmp(argv[1], "status")) {
 		action = ACTION_STATUS;
 		quiet = 1;
@@ -208,6 +219,7 @@ int main(int argc, char *argv[], char *envp[])
 		action = ACTION_QUOTAOFF;
 	} else if (!strcmp(argv[1], "quotainit")) {
 		action = ACTION_QUOTAINIT;
+#ifdef HAVE_PLOOP
 	} else if (!strcmp(argv[1], "snapshot")) {
 		action = ACTION_SNAPSHOT_CREATE;
 	} else if (!strcmp(argv[1], "snapshot-switch")) {
@@ -216,6 +228,7 @@ int main(int argc, char *argv[], char *envp[])
 		action = ACTION_SNAPSHOT_DELETE;
 	} else if (!strcmp(argv[1], "snapshot-list")) {
 		action = ACTION_SNAPSHOT_LIST;
+#endif
 	} else if (!strcmp(argv[1], "--help")) {
 		usage(0);
 	} else {
